@@ -1,4 +1,3 @@
-from threading import Thread
 import urllib.request, urllib.error, urllib.parse
 import os
 import datetime
@@ -7,10 +6,12 @@ import time
 import logging
 import traceback
 import pickle
-from functools import wraps
+import mysql.connector
 import socket, struct
-from zeroconf import ServiceBrowser, Zeroconf, IPVersion
 
+from threading import Thread
+from functools import wraps
+from zeroconf import ServiceBrowser, Zeroconf, IPVersion
 from ethoscope_node.utils.etho_db import ExperimentalDB
 from ethoscope_node.utils.mysql_backup import db_diff
 
@@ -210,6 +211,7 @@ class Ethoscope(Device):
             'static' : "static",
             'controls' : "controls",
             'machine_info' : "machine",
+            'connected_module' : "module",
             'update' : "update",
             'dumpdb' : "dumpSQLdb"
             }
@@ -224,7 +226,8 @@ class Ethoscope(Device):
                                      "restart" : ["stopped"],
                                      "dumpdb" : ["stopped"],
                                      "offline": [],
-                                     "convertvideos" : ["stopped"]
+                                     "convertvideos" : ["stopped"],
+                                     "test_module" : ["stopped"]
                                      }
 
     def __init__(self, ip, port = ETHOSCOPE_PORT, refresh_period = 5, results_dir = "/ethoscope_data/results"):
@@ -342,7 +345,17 @@ class Ethoscope(Device):
             return out
         else:
             return {}
-        
+
+    def connected_module(self):
+        '''
+        Asks the ethoscope to give us information about connected modules. We do not do this through machine info because the step requires
+        opening a serial connection with a module and we do not want to risk doing this when the module is actually running in an experiment.
+        '''
+        if self._id:
+            url = "http://%s:%i/%s/%s" % (self._ip, self._port, self._remote_pages['connected_module'], self._id)
+            return self._get_json(url, timeout=12)
+        else:
+            return {}
 
     def skip_scanning(self, value):
         self._skip_scanning = value
@@ -408,7 +421,6 @@ class Ethoscope(Device):
     def last_image(self):
         """
         Collects the last drawn image fromt the device
-        TODO: on the device side, this should not rely on an actuale image file but be fished from memory
         """
         # we return none if the device is not in a stoppable status (e.g. running, recording)
         if self._info["status"] not in self._allowed_instructions_status["stop"]:
@@ -605,8 +617,6 @@ class Ethoscope(Device):
 
             try:
                 logging.warning("No information regarding backup file from the ethoscope") 
-                import mysql.connector
-                from ethoscope.utils.io import SQL_CHARSET
                 
                 device_id = self._info["id"]
                 device_name = self._info["name"]
@@ -618,7 +628,7 @@ class Ethoscope(Device):
                                                    connect_timeout=timeout,
                                                    **self._ethoscope_db_credentials,
                                                    buffered=True,
-                                                   charset=SQL_CHARSET,
+                                                   charset='latin1', #this should match what set in ethoscope.utils.io
                                                    use_unicode=True)
                 cur = mysql_db.cursor()
                 cur.execute(com)
@@ -637,9 +647,9 @@ class Ethoscope(Device):
                                               )
 
             except Exception as e:
-                #logging.error("Could not generate backup path for device. Probably a MySQL issue")
+                logging.error("Could not generate backup path for device. Probably a MySQL issue")
                 #logging.error(traceback.format_exc())
-                 output_db_file = "None"
+                output_db_file = "None"
 
 
         self._info.update( {"backup_path": output_db_file} )
