@@ -6,10 +6,163 @@ import os
 import logging
 import traceback
 
+SQL_CHARSET = 'latin1'
+
 class DBNotReadyError(Exception):
     pass
 
-class MySQLdbToSQlite(object):
+class baseSQLconnector():
+
+    """
+    Basic class providing functions to compare the status of a local SQLlite3 db to the remote counterpart
+    This is used to check if the db backup is in good shape
+    
+    The SQL command can be run from commandline for debugging purposes:
+    
+    mysql -u ethoscope -p -h 192.168.1.45 -e 'SELECT table_name,table_rows FROM INFORMATION_SCHEMA.tables WHERE table_schema LIKE "ETHOSCOPE%";'
+    """
+
+    def _get_remote_db_info(self):
+        """
+        This is fast but not reliable when the databases uses InnoDB because the information schema does not necessarily gets updated.
+        One would have to analyse table by table first
+        see: https://bugs.mysql.com/bug.php?id=99022
+        """
+        src = mysql.connector.connect(host=self._remote_host,
+                                      user=self._remote_user,
+                                      passwd=self._remote_pass,
+                                      buffered=True,
+                                      charset=SQL_CHARSET,
+                                      use_unicode=True)
+            
+        src_cur = src.cursor(buffered=True)
+        
+        command = 'SELECT table_name, table_rows FROM INFORMATION_SCHEMA.tables WHERE table_schema LIKE "ETHOSCOPE%";'
+        src_cur.execute(command)
+        tables = src_cur.fetchall()
+        
+        src.close()
+        
+        return {a[0] : a[1] for a in tables}
+
+    def _get_remote_db_info_slow(self):
+        """
+        """
+
+        #fetches data about the size of the remote db ( remote_local_tables_dictionary )
+        src = mysql.connector.connect(host=self._remote_host,
+                                      user=self._remote_user,
+                                      passwd=self._remote_pass,
+                                      buffered=True,
+                                      charset=SQL_CHARSET,
+                                      use_unicode=True)
+            
+        src_cur = src.cursor(buffered=True)
+        
+        command = 'SELECT TABLE_SCHEMA, TABLE_NAME FROM information_schema.tables WHERE TABLE_SCHEMA LIKE "ETHOSCOPE%";'
+        src_cur.execute(command)
+        tables = src_cur.fetchall()
+        
+        remote_local_tables_dictionary = {dbn : {} for dbn in set([entry[0] for entry in tables])}
+        
+        for entry in tables: 
+            db_name = entry[0]
+            table_name = entry[1]
+            
+            if table_name in ["ROI_MAP", "VAR_MAP"] or table_name.startswith("METADATA"):
+                #tables that do not have a unique id - slower command
+                command = 'SELECT count(*) from %s.%s' % (db_name, table_name)
+            else:
+                #tables that do
+                command = 'SELECT max(id) FROM %s.%s' % (db_name, table_name)
+            
+            src_cur.execute(command)
+            remote_local_tables_dictionary [db_name] . update ( { table_name :  src_cur.fetchone()[0] } )
+
+        src.commit()
+        src.close()
+        
+        return remote_local_tables_dictionary
+        #return remote_local_tables_dictionary[db_name]
+
+    def _get_local_db_info(self):
+        """
+        """
+
+        local_tables_dictionary = {}
+        
+        # now, this is very funny: this returns false if the folder is mounted via ssh - wtf?
+        if os.path.exists(self._dst_path):
+            
+            #connection to the local node file requires no credentials
+            with sqlite3.connect(self._dst_path, check_same_thread=False) as dst:
+                dst_cur = dst.cursor()
+                command = 'SELECT name FROM sqlite_master WHERE type ="table" AND name NOT LIKE "sqlite_%";'
+                dst_cur.execute(command)
+                tables = dst_cur.fetchall()
+                
+                for entry in tables: 
+                    table_name = entry[0]
+                    
+                    if table_name not in ["ROI_MAP", "VAR_MAP", "METADATA"]:
+                        command = 'SELECT max(id) FROM %s;' % table_name
+                    else:
+                        command = 'SELECT count(*) from %s' % table_name
+                    
+                    dst_cur.execute(command)
+                    local_tables_dictionary . update ( { table_name :  dst_cur.fetchone()[0] } )            
+                
+            return local_tables_dictionary
+        
+        else:
+            
+            return {} # sqlite3 file does not exist yet
+            
+    def compare_databases(self, use_fast_mode=False):
+        """
+        """
+        total_remote = 0
+        total_local = 0
+        
+        try:
+            if use_fast_mode:
+                remote_tables_info = self._get_remote_db_info()
+            else:
+                remote_tables_info = self._get_remote_db_info_slow()
+               
+        except:
+            logging.error("Problem getting info from the remote database: %s " % self._remote_db_name)
+        
+        try:
+            local_tables_info = self._get_local_db_info()
+        except:
+            logging.error("Problem getting info from the local database %s - perhaps it is locked?" % self._dst_path)
+        
+        #try:
+        for table in sorted(local_tables_info):
+
+            l = local_tables_info[table]
+
+            if use_fast_mode:
+                r = remote_tables_info[table] # the fast system
+            else:
+                r = remote_tables_info[self._remote_db_name][table] # the slow system
+            
+            if r == None : r = 0
+            if l == None : l = 0
+            
+            total_remote += int(r)
+            total_local += int(l)
+            
+            #print ("Transferred %s / %s for table %s (%0.2f)" % (l, r, table, l/r*100))
+        if total_remote == 0: return -1
+        else: return total_local/total_remote*100
+
+        #except:
+        #    return -1
+
+
+class MySQLdbToSQlite(baseSQLconnector):
     _max_n_rows_to_insert = 10000
 
     def __init__(self,
@@ -44,7 +197,10 @@ class MySQLdbToSQlite(object):
                                       user=self._remote_user,
                                       passwd=self._remote_pass,
                                       db=self._remote_db_name,
-                                      connect_timeout=45)
+                                      connect_timeout=45,
+                                      buffered=True,
+                                      charset=SQL_CHARSET,
+                                      use_unicode=True)
 
 
         self._dst_path=dst_path
@@ -134,101 +290,6 @@ class MySQLdbToSQlite(object):
         else:
             self._replace_table(table_name, src, dst, dump_in_csv)
 
-    def _get_remote_db_info(self):
-        """
-        """
-        #fetches data about the size of the remote db ( remote_local_tables_dictionary )
-        src = mysql.connector.connect(host=self._remote_host,
-                                      user=self._remote_user,
-                                      passwd=self._remote_pass)
-            
-        src_cur = src.cursor(buffered=True)
-        
-        command = 'SELECT TABLE_SCHEMA, TABLE_NAME FROM information_schema.tables WHERE TABLE_SCHEMA LIKE "ETHOSCOPE%";'
-        src_cur.execute(command)
-        tables = src_cur.fetchall()
-        
-        remote_local_tables_dictionary = {dbn : {} for dbn in set([entry[0] for entry in tables])}
-        
-        for entry in tables: 
-            db_name = entry[0]
-            table_name = entry[1]
-            
-            if table_name not in ["ROI_MAP", "VAR_MAP", "METADATA"]:
-                command = 'SELECT max(id) FROM %s.%s' % (db_name, table_name)
-            else:
-                command = 'SELECT count(*) from %s.%s' % (db_name, table_name)
-            
-            src_cur.execute(command)
-            remote_local_tables_dictionary [db_name] . update ( { table_name :  src_cur.fetchone()[0] } )
-
-        src.commit()
-        src.close()
-        
-        return remote_local_tables_dictionary
-
-    def _get_local_db_info(self):
-        """
-        """
-        local_tables_dictionary = {}
-        
-        with sqlite3.connect(self._dst_path, check_same_thread=False) as dst:
-            dst_cur = dst.cursor()
-            command = 'SELECT name FROM sqlite_master WHERE type ="table" AND name NOT LIKE "sqlite_%";'
-            dst_cur.execute(command)
-            tables = dst_cur.fetchall()
-
-
-            
-            for entry in tables: 
-                table_name = entry[0]
-                
-                if table_name not in ["ROI_MAP", "VAR_MAP", "METADATA"]:
-                    command = 'SELECT max(id) FROM %s;' % table_name
-                else:
-                    command = 'SELECT count(*) from %s' % table_name
-                
-                dst_cur.execute(command)
-                local_tables_dictionary . update ( { table_name :  dst_cur.fetchone()[0] } )            
-            
-        return local_tables_dictionary
-        
-    def compare_databases(self):
-        """
-        """
-        total_remote = 0
-        total_local = 0
-        
-        try:
-            remote_tables_info = self._get_remote_db_info()
-        except:
-            logging.error("Problem getting info from the remote database")
-        
-        try:
-            local_tables_info = self._get_local_db_info()
-        except:
-            logging.error("Problem getting info from the local database %s" % self._dst_path)
-        
-        try:
-            for table in sorted(local_tables_info):
-                l = local_tables_info[table]
-                r = remote_tables_info[self._remote_db_name][table]
-                
-                if r == None : r = 0
-                if l == None : l = 0
-                
-                total_remote += int(r)
-                total_local += int(l)
-                
-                #print ("Transferred %s / %s for table %s (%0.2f)" % (l, r, table, l/r*100))
-                
-            return total_local/total_remote*100
-
-        except:
-            return -1
-
-        
-            
     def update_roi_tables(self):
         """
         Fetch new ROI tables and new data points in the remote and use them to update local db
@@ -238,7 +299,10 @@ class MySQLdbToSQlite(object):
         src = mysql.connector.connect(host=self._remote_host,
                                       user=self._remote_user,
                                       passwd=self._remote_pass,
-                                      db=self._remote_db_name)
+                                      db=self._remote_db_name,
+                                      buffered=True,
+                                      charset=SQL_CHARSET,
+                                      use_unicode=True)
 
         with sqlite3.connect(self._dst_path, check_same_thread=False) as dst:
 
@@ -420,128 +484,23 @@ class MySQLdbToSQlite(object):
             dst_cur.execute(command, args)
             dst.commit()
 
-
-class db_diff():
+class db_diff(baseSQLconnector):
     """
     Class used to compare the status of a local SQLlite3 db to the remote counterpart
     This is used to check if the db backup is in good shape
-    The same functions are duplicated in node_src/ethoscope_node/utils/mysql_backup.py
     """
 
-    _remote_user = "node"
-    _remote_pass = "node"
+    _remote_user = "ethoscope"
+    _remote_pass = "ethoscope"
 
     def __init__(self, db_name, remote_host, filename):
         """
         remote_host is the IP address of the ethoscope we are supposed to check on
+        db_name is the name of the remote database
+
         filename is the local SQLlite3 file to check
-        db_name is the name of the database
         """
     
         self._remote_host = remote_host
         self._dst_path = filename
         self._remote_db_name = db_name
-
-    
-    def _get_remote_db_info(self):
-        """
-        """
-
-        #fetches data about the size of the remote db ( remote_local_tables_dictionary )
-        src = mysql.connector.connect(host=self._remote_host,
-                                      user=self._remote_user,
-                                      passwd=self._remote_pass)
-            
-        src_cur = src.cursor(buffered=True)
-        
-        command = 'SELECT TABLE_SCHEMA, TABLE_NAME FROM information_schema.tables WHERE TABLE_SCHEMA LIKE "ETHOSCOPE%";'
-        src_cur.execute(command)
-        tables = src_cur.fetchall()
-        
-        remote_local_tables_dictionary = {dbn : {} for dbn in set([entry[0] for entry in tables])}
-        
-        for entry in tables: 
-            db_name = entry[0]
-            table_name = entry[1]
-            
-            if table_name in ["ROI_MAP", "VAR_MAP"] or table_name.startswith("METADATA"):
-                #tables that do not have a unique id - slower command
-                command = 'SELECT count(*) from %s.%s' % (db_name, table_name)
-            else:
-                #tables that do
-                command = 'SELECT max(id) FROM %s.%s' % (db_name, table_name)
-            
-            src_cur.execute(command)
-            remote_local_tables_dictionary [db_name] . update ( { table_name :  src_cur.fetchone()[0] } )
-
-        src.commit()
-        src.close()
-        
-        return remote_local_tables_dictionary
-
-    def _get_local_db_info(self):
-        """
-        """
-
-        local_tables_dictionary = {}
-        
-        if os.path.exists(self._dst_path):
-        
-            with sqlite3.connect(self._dst_path, check_same_thread=False) as dst:
-                dst_cur = dst.cursor()
-                command = 'SELECT name FROM sqlite_master WHERE type ="table" AND name NOT LIKE "sqlite_%";'
-                dst_cur.execute(command)
-                tables = dst_cur.fetchall()
-
-
-                
-                for entry in tables: 
-                    table_name = entry[0]
-                    
-                    if table_name not in ["ROI_MAP", "VAR_MAP", "METADATA"]:
-                        command = 'SELECT max(id) FROM %s;' % table_name
-                    else:
-                        command = 'SELECT count(*) from %s' % table_name
-                    
-                    dst_cur.execute(command)
-                    local_tables_dictionary . update ( { table_name :  dst_cur.fetchone()[0] } )            
-                
-            return local_tables_dictionary
-        
-        else:
-            
-            return {} # sqlite3 file does not exist yet
-            
-    def compare_databases(self):
-        """
-        """
-        total_remote = 0
-        total_local = 0
-        
-        try:
-            remote_tables_info = self._get_remote_db_info()
-        except:
-            logging.error("Problem getting info from the remote database: %s " % self._remote_db_name)
-        
-        try:
-            local_tables_info = self._get_local_db_info()
-        except:
-            logging.error("Problem getting info from the local database %s - perhaps it is locked?" % self._dst_path)
-        
-        try:
-            for table in sorted(local_tables_info):
-                l = local_tables_info[table]
-                r = remote_tables_info[self._remote_db_name][table]
-                
-                if r == None : r = 0
-                if l == None : l = 0
-                
-                total_remote += int(r)
-                total_local += int(l)
-                
-                #print ("Transferred %s / %s for table %s (%0.2f)" % (l, r, table, l/r*100))
-                
-            return total_local/total_remote*100
-
-        except:
-            return -1
